@@ -66,6 +66,7 @@ def ensure_downloaded(domain, filename, advance=None):
 def isstring(var):
     return isinstance(var, Orange.data.StringVariable)
 
+
 def listAvailable():
     taxids = taxonomy.common_taxids()
     essential = [(taxonomy.name(taxid), 'gene_association.{}'.format(taxid)) for taxid in taxids
@@ -296,7 +297,7 @@ class OWGOBrowser(widget.OWWidget):
 
         self.listView.header().setSectionsClickable(True)
         self.listView.header().setSortIndicatorShown(True)
-        self.listView.header().setSortIndicator(self.DAGcolumns.index('p-value'), Qt.DescendingOrder)
+        self.listView.header().setSortIndicator(self.DAGcolumns.index('p-value'), Qt.AscendingOrder)
         self.listView.setSortingEnabled(True)
         self.listView.setItemDelegateForColumn(
             6, EnrichmentColumnItemDelegate(self))
@@ -310,7 +311,7 @@ class OWGOBrowser(widget.OWWidget):
         self.sigTerms.setHeaderLabels(self.DAGcolumns)
         self.sigTerms.setSortingEnabled(True)
         self.sigTerms.setSelectionMode(QTreeView.ExtendedSelection)
-        self.sigTerms.header().setSortIndicator(self.DAGcolumns.index('p-value'), Qt.DescendingOrder)
+        self.sigTerms.header().setSortIndicator(self.DAGcolumns.index('p-value'), Qt.AscendingOrder)
         self.sigTerms.setItemDelegateForColumn(
             6, EnrichmentColumnItemDelegate(self))
 
